/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.scan.impl;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.ow2.util.asm.ClassReader;
import org.ow2.util.scan.api.IClassesLocator;
import org.ow2.util.scan.api.IScanner;
import org.ow2.util.scan.api.IScannerResult;
import org.ow2.util.scan.api.ScanException;
import org.ow2.util.scan.api.configurator.ISessionConfigurator;

/**
 * ArchiveScanner.
 * This scanner use ASM to analyze classes.
 * @author Gael Lalire
 */
public final class ASMScannerImpl implements IScanner {

    /**
     * @param url the class url
     * @param sessionConfigurator the configurator for this session
     * @throws ScanException if error occurs
     * @return the scan result
     */
    @Override
    public IScannerResult scanClass(URL url, ISessionConfigurator sessionConfigurator) throws ScanException {
        ScannerResult scannerResult = new ScannerResult();
        InputStream is = null;
        try {
            URLConnection urlConnection = url.openConnection();
            urlConnection.setDefaultUseCaches(false);
            is = urlConnection.getInputStream();
            new ClassReader(is).accept(new ScanClassVisitor(sessionConfigurator, scannerResult), 0);
        } catch (Exception ioe) {
            throw new ScanException("Error while analyzing file entry '" + url + "'", ioe);
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    throw new ScanException("Error while closing input stream of the entry '" + url + "'", e);
                }
            }
        }
        return scannerResult;
    }


    /**
     * @param initialClassNameSet initial set of classes to scan at the beginning
     * @param archiveClassLocator class locator for current archive to scan
     * @param externClassLocator class locator list for external libraries
     *        (accessibles classes which are not in archive)
     * @param archiveConfigurator a configurator (provide metadata creation
     *        method and list of annotation to analyze)
     * @throws ScanException if error occurs
     */
    @Override
    public void scan(final Set<String> initialClassNameSet, final IClassesLocator archiveClassLocator,
            final List<? extends IClassesLocator> externalClassLocator, final ISessionConfigurator sessionConfigurator)
            throws ScanException {
        Set<String> classNameSet = new HashSet<String>();
        List<String> scannedClasses = new ArrayList<String>();
        List<String> nextClassNameToScan = new ArrayList<String>();
        nextClassNameToScan.addAll(initialClassNameSet);

        while (nextClassNameToScan.size() != 0) {
            for (String className : nextClassNameToScan) {
                URL url = archiveClassLocator.getClassURL(className);
                if (url == null && externalClassLocator != null) {
                    for (IClassesLocator classesLocator : externalClassLocator) {
                        url = classesLocator.getClassURL(className);
                        if (url != null) {
                            break;
                        }
                    }
                }
                IScannerResult result = null;
                if (url != null) {
                    result = scanClass(url, sessionConfigurator);
                }
                Collection<String> newClasses = result.getReferencedClasses();
                if (newClasses != null) {
                    classNameSet.addAll(newClasses);
                }

                // we add to scanned even if there was no scan (class not
                // found)
                scannedClasses.add(className);
            }

            // keep what is not scanned
            nextClassNameToScan.clear();
            for (String className : classNameSet) {
                if (!scannedClasses.contains(className)) {
                    nextClassNameToScan.add(className);
                }
            }

            // recalculate classNameSet
            classNameSet.clear();
        }
    }

}
