/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.scan.impl;

import java.lang.annotation.ElementType;
import java.util.List;

import org.ow2.util.asm.ClassVisitor;
import org.ow2.util.asm.FieldVisitor;
import org.ow2.util.asm.MethodVisitor;
import org.ow2.util.scan.api.configurator.IClassConfigurator;
import org.ow2.util.scan.api.configurator.IFieldConfigurator;
import org.ow2.util.scan.api.configurator.IMethodConfigurator;
import org.ow2.util.scan.api.configurator.ISessionConfigurator;
import org.ow2.util.scan.api.configurator.IViewConfigurator;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IFieldMetadata;
import org.ow2.util.scan.api.metadata.IMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;
import org.ow2.util.scan.api.metadata.structures.IClass;
import org.ow2.util.scan.api.metadata.structures.IField;
import org.ow2.util.scan.api.metadata.structures.IMethod;
import org.ow2.util.scan.impl.metadata.ClassMetadata;
import org.ow2.util.scan.impl.metadata.FieldMetadata;
import org.ow2.util.scan.impl.metadata.JClass;
import org.ow2.util.scan.impl.metadata.JField;
import org.ow2.util.scan.impl.metadata.JMethod;
import org.ow2.util.scan.impl.metadata.MethodMetadata;

/**
 * ScanClassVisitor.
 * @author Florent Benoit
 * @author Gael Lalire
 */
public final class ScanClassVisitor extends ScanCommonVisitor implements ClassVisitor {

    /**
     * Class Metadata analyzed during this parsing.
     */
    private IClassMetadata classMetadata;

    /**
     * Class Configurator.
     */
    private IClassConfigurator classConfigurator;

    /**
     * Constructor.
     * @param archiveConfigurators list of archives
     * @param referencedClasses class name referenced by metadata
     */
    public ScanClassVisitor(final ISessionConfigurator sessionConfigurator, ScannerResult scannerResult) {
        super(sessionConfigurator, scannerResult, ElementType.TYPE);
    }

    /**
     * visit the class.
     * @param version the version
     * @param access access (public, ...)
     * @param name class name
     * @param signature signature
     * @param superName super class name
     * @param interfaces implemented interfaces name
     */
    @Override
    public void visit(final int version,
                      final int access,
                      final String name,
                      final String signature,
                      final String superName,
                      final String[] interfaces) {
        IClass jClass = new JClass(version, access, name, signature, superName, interfaces);

        // Build a metadata around this JClass object
        this.classMetadata = new ClassMetadata();
        setViewConfigurators(classMetadata);
        this.classMetadata.setJClass(jClass);
        setMetadata(classMetadata);
        this.classConfigurator = getSessionConfigurator().createClassConfigurator(classMetadata);
    }



    /**
     * @param access access of method (public, ...)
     * @param name name of method
     * @param desc desc
     * @param signature signature of method
     * @param exceptions throwed exceptions
     * @return a visitor
     */
    @Override
    public MethodVisitor visitMethod(final int access, final String name, final String desc, final String signature,
            final String[] exceptions) {
        IMethod jMethod = new JMethod(access, name, desc, signature, exceptions);

        // Build a metadata around this jField object
        IMethodMetadata methodMetadata = new MethodMetadata(jMethod, classMetadata);
        setViewConfigurators(methodMetadata);

        IMethodConfigurator methodConfigurator = classConfigurator.createMethodConfigurator(methodMetadata);
        return new ScanMethodVisitor(getSessionConfigurator(), methodConfigurator, methodMetadata, getScannerResult());
    }

    /**
     * @param access access of field (public, ...)
     * @param name name of field
     * @param desc desc
     * @param signature signature of field
     * @param value value of field
     * @return a visitor
     */
    @Override
    public FieldVisitor visitField(final int access, final String name, final String desc, final String signature,
            final Object value) {
        IField jField = new JField(access, name, desc, signature, value);

        // Build a metadata around this jField object
        IFieldMetadata fieldMetadata = new FieldMetadata(jField, classMetadata);
        setViewConfigurators(fieldMetadata);

        IFieldConfigurator fieldConfigurator = classConfigurator.createFieldConfigurator(fieldMetadata);

        return new ScanFieldVisitor(getSessionConfigurator(), fieldConfigurator, fieldMetadata, getScannerResult());
    }


    /**
     * call configurationComplete on each configurator.
     */
    @Override
    public void visitEnd() {
        classConfigurator.configurationComplete(getScannerResult());
    }


    /**
     * Sets the view configurators on the given metadata
     * @param metadata
     */
    protected void setViewConfigurators(IMetadata metadata) {
        List<IViewConfigurator> viewConfigurators = getSessionConfigurator().getViewConfigurators();
        if (viewConfigurators != null) {
            for (IViewConfigurator viewConfigurator : viewConfigurators) {
                metadata.addViewConfigurator(viewConfigurator);
            }
        }
    }
}
