/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.scan.impl.metadata;

import java.lang.annotation.ElementType;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.util.scan.api.metadata.CommonMetadata;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IFieldMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;
import org.ow2.util.scan.api.metadata.structures.IClass;
import org.ow2.util.scan.api.metadata.structures.IField;
import org.ow2.util.scan.api.metadata.structures.IMember;
import org.ow2.util.scan.api.metadata.structures.IMethod;


/**
 * Defines the class metadata used to define a class
 * @author Florent Benoit
 * @author Gael Lalire
 */
public class ClassMetadata extends CommonMetadata implements IClassMetadata {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = 1392857823357889714L;

    /**
     * Class.
     */
    private IClass jClass;

    /**
     * List of method annotations metadata.
     */
    private Map<IMethod, IMethodMetadata> methodsAnnotationMetadata = null;

    /**
     * List of field annotations metadata.
     */
    private Map<IField, IFieldMetadata> fieldsAnnotationMetadata = null;


    /**
     * No-arg Constructor.
     */
    public ClassMetadata() {
        this.methodsAnnotationMetadata = new HashMap<>();
        this.fieldsAnnotationMetadata = new HashMap<>();
    }

    /**
     * add a method metadata.
     * @param methodAnnotationMetadata the method metadata
     */
    @Override
    public void addMethodMetadata(final IMethodMetadata methodAnnotationMetadata) {
        IMethod key = methodAnnotationMetadata.getJMethod();
        // No check: the same method can be added from the super class (ie : super = private, current = private)
        // So it means that there is no overriding
        methodsAnnotationMetadata.put(key, methodAnnotationMetadata);
    }

    /**
     * @param jMethod key of the map of methods annotations.
     * @return method annotation metadata of a given method.
     */
    @Override
    public final IMethodMetadata getMethodMetadata(final IMethod jMethod) {
        return methodsAnnotationMetadata.get(jMethod);
    }

    /**
     * @param methodName the name of the method to search
     * @return list of method annotation metadata for the given method name
     */
    @Override
    public final List<IMethodMetadata> searchMethodMetadata(final String methodName) {
        if (methodName == null) {
            throw new IllegalArgumentException("Cannot search null methods");
        }
        List<IMethodMetadata> list = new ArrayList<>();
        for (IMethodMetadata method : methodsAnnotationMetadata.values()) {
            if (methodName.equals(method.getJMethod().getName())) {
                list.add(method);
            }
        }
        return list;
    }

    /**
     * @param fieldName the name of the field to search
     * @return list of field annotation metadata for the given field name
     */
    @Override
    public final List<IFieldMetadata> searchFieldMetadata(final String fieldName) {
        if (fieldName == null) {
            throw new IllegalArgumentException("Cannot search null fields");
        }
        List<IFieldMetadata> list = new ArrayList<>();
        for (IFieldMetadata field : fieldsAnnotationMetadata.values()) {
            if (fieldName.equals(field.getJField().getName())) {
                list.add(field);
            }
        }
        return list;
    }


    /**
     * Get collections of methods annotation metadata.
     * @return collections of methods annotation metadata.
     */
    @Override
    public final Collection<IMethodMetadata> getMethodMetadataCollection() {
        return methodsAnnotationMetadata.values();
    }


    /**
     * Add field annotation metadata for a given Bean.
     * @param fieldAnnotationMetadata metadata of a field.
     */
    @Override
    public void addFieldMetadata(final IFieldMetadata fieldAnnotationMetadata) {
        IField key = fieldAnnotationMetadata.getJField();
        // already exists ?
        if (fieldsAnnotationMetadata.containsKey(key)) {
            throw new IllegalStateException();
        }
        fieldsAnnotationMetadata.put(key, fieldAnnotationMetadata);
    }

    /**
     * @param jField key of the map of fields annotations.
     * @return field annotation metadata of a given method.
     */
    @Override
    public IFieldMetadata getFieldMetadata(final IField jField) {
        return fieldsAnnotationMetadata.get(jField);
    }

    /**
     * Get collections of fields annotation metadata.
     * @return collections of fields annotation metadata.
     */
    @Override
    public final Collection<IFieldMetadata> getFieldMetadataCollection() {
        return fieldsAnnotationMetadata.values();
    }


    /**
     * @return class representation
     */
    @Override
    public final IClass getJClass() {
        return jClass;
    }


    /**
     * @param jClass jClass
     */
    @Override
    public final void setJClass(final IClass jClass) {
        this.jClass = jClass;
    }

    /**
     * Return class name (with dot)
     */
    @Override
    public String getClassName() {
        return jClass.getName().replace("/", ".");
    }

    /**
     * Return class name (with slash)
     */
    @Override
    public String getInternalClassName() {
        return jClass.getName();
    }

    @Override
    protected ElementType getElementType() {
        return ElementType.TYPE;
    }

    @Override
    public IMember getMember() {
        return jClass;
    }

    @Override
    public String getSuperName() {
        return jClass.getSuperName();
    }

}
