/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.scan.impl.metadata;

import java.util.Arrays;

import org.ow2.util.scan.api.metadata.structures.IClass;

/**
 * ASM class representation.
 * @author Gael Lalire
 */
public final class JClass extends JMember implements IClass {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = -977433124031808986L;

    /**
     * version.
     */
    private final int version;

    /**
     * superName.
     */
    private final String superName;

    /**
     * interfaces.
     */
    private String[] interfaces;

    /**
     * Constructor.
     * @param version version
     * @param access access
     * @param name name
     * @param signature signature
     * @param superName superName
     * @param interfaces interfaces
     */
    public JClass(final int version, final int access, final String name, final String signature, final String superName,
            final String[] interfaces) {
        this.version = version;
        setAccess(access);
        if (name == null) {
            throw new IllegalArgumentException("Name argument of JClass constructor can not be null");
        }
        setName(name);
        setSignature(signature);
        if (superName == null) {
            throw new IllegalArgumentException("SuperName argument of JClass constructor can not be null");
        }
        this.superName = superName;
        String[] tmpInterfaces = interfaces;
        if (tmpInterfaces != null) {
            this.interfaces = tmpInterfaces.clone();
        } else {
            this.interfaces = null;
        }
    }



    /**
     * @return version
     */
    @Override
    public int getVersion() {
        return version;
    }



    /**
     * @return superName
     */
    @Override
    public String getSuperName() {
        return superName;
    }

    /**
     * @return interfaces
     */
    @Override
    public String[] getInterfaces() {
        if (interfaces != null) {
            return interfaces.clone();
        }
        return null;
    }



    /**
     * @param obj the object to compare
     * @return true if equals
     */
    @Override
    public boolean equals(final Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof JClass)) {
            return false;
        } else {
            JClass jClass = (JClass) obj;
            if (version == jClass.version && getAccess() == jClass.getAccess() && getName().equals(jClass.getName())
                    && superName.equals(jClass.superName)
                    && Arrays.deepEquals(interfaces, jClass.interfaces)) {
                return true;
            }
            return false;
        }
    }

    /**
     * Returns a string representation of the class.
     * @return a string representation of the object.
     */
    @Override
    public String toString() {
        return getName().replace('/', '.');
    }
}
