/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.scan.impl.metadata;

import java.util.Arrays;

import org.ow2.util.scan.api.metadata.structures.IMethod;

/**
 * This class defines a Method object. It is not based on reflection but allows
 * to build a JMethod based on java.lang.reflect.Method
 * @author Florent Benoit
 */
public final class JMethod extends JMember implements IMethod {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = -1003811648237672235L;

    /**
     * Method's descriptor.
     */
    private String descriptor = null;

    /**
     * Exceptions of the method.
     */
    private final String[] exceptions;

    /**
     * Constructor.
     * @param access the access mode (see {@link org.ow2.easybeans.asm.Opcodes})
     * @param name the method's name.
     * @param descriptor the method's descriptor (see
     *        {@link org.ow2.util.scan.api.IType Type}).
     * @param signature the method's signature. May be <tt>null</tt> if the
     *        method parameters, return type and exceptions do not use generic
     *        types.
     * @param exceptions the internal names of the method's exception classes
     *        (see
     *        {@link org.ow2.util.scan.api.IType#getInternalName() getInternalName}).
     *        May be <tt>null</tt>.
     */
    public JMethod(final int access, final String name, final String descriptor, final String signature,
            final String[] exceptions) {
        setAccess(access);
        setName(name);
        this.descriptor = descriptor;
        setSignature(signature);
        String[] tmpExceptions = exceptions;
        if (tmpExceptions != null) {
            this.exceptions = tmpExceptions.clone();
        } else {
            this.exceptions = null;
        }
    }

    /**
     * Indicates whether some other object is "equal to" this one.
     * @param obj object to compare
     * @return true if given object is equals
     */
    @Override
    public boolean equals(final Object obj) {
        if (obj != null && obj instanceof JMethod) {
            JMethod other = (JMethod) obj;

            // same name
            if (!this.getName().equals(other.getName())) {
                return false;
            }

            // same descriptor
            if ((this.descriptor != null) && (!this.descriptor.equals(other.descriptor))) {
                return false;
            }

            // Don't check signature (which include generics information)
            // For example void method(List<Integer>) and
            //             void method(List<String>)
            // and even if signature is different, they have same erasure
            // This is not allowed, so don't need to check signature information.


            // if all tests succeed, return true
            return true;
        }
        return false;
    }


    /**
     * @return method descriptor
     */
    @Override
    public String getDescriptor() {
        return descriptor;
    }

    /**
     * @return method exceptions
     */
    @Override
    public String[] getExceptions() {
        if (exceptions != null) {
            return exceptions.clone();
        }
        return null;
    }

    /**
     * @return string representation
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        // classname
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));

        // name
        sb.append("[name=");
        sb.append(getName());

        // access
        sb.append(", access=");
        sb.append(getAccess());

        // descriptor
        if (descriptor != null) {
            sb.append(", descriptor=");
            sb.append(descriptor);
        }

        // signature
        if (getSignature() != null) {
            sb.append(", signature=");
            sb.append(getSignature());
        }

        // exceptions
        if (exceptions != null) {
            sb.append(", exceptions=");
            sb.append(Arrays.asList(exceptions));
        }
        sb.append("]");
        return sb.toString();
    }

}
