/**
 * Copyright 2006-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.scan.impl.metadata;

import java.lang.annotation.ElementType;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.ow2.util.asm.Type;
import org.ow2.util.scan.api.metadata.CommonMetadata;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;
import org.ow2.util.scan.api.metadata.structures.IMember;
import org.ow2.util.scan.api.metadata.structures.IMethod;

/**
 * This class represents the annotation metadata of a method.
 * @author Florent Benoit
 */
public class MethodMetadata extends CommonMetadata implements IMethodMetadata {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = 1006697917121678878L;

    /**
     * Method on which we got metadata.
     */
    private IMethod jMethod = null;

    /**
     * Parent metadata.
     */
    private IClassMetadata classMetadata = null;

    /**
     * Constructor.
     * @param jMethod the method on which we will set/add metadata
     * @param classMetadata the parent metadata.
     */
    public MethodMetadata(final IMethod jMethod, final IClassMetadata classMetadata) {
        this.jMethod = jMethod;
        this.classMetadata = classMetadata;
        setParent(classMetadata);
    }

    /**
     * @return JMethod object
     */
    @Override
    public IMethod getJMethod() {
        return this.jMethod;
    }

    /**
     * Sets the JMethod object (for changing some data).
     * @param jMethod the given JMethod object
     */
    @Override
    public void setJMethod(final IMethod jMethod) {
        this.jMethod = jMethod;
    }

    /**
     * @return parent metadata (class)
     */
    @Override
    public IClassMetadata getClassMetadata() {
        return classMetadata;
    }

    /**
     * @param classMetadata parent metadata (class)
     */
    @Override
    public void setClassMetadata(final IClassMetadata classMetadata) {
        this.classMetadata = classMetadata;
    }

    @Override
    protected ElementType getElementType() {
        return ElementType.METHOD;
    }

    @Override
    public IMember getMember() {
        return getJMethod();
    }

    @Override
    public String[] getParametersClassName() {
        Type[] argumentTypes = Type.getArgumentTypes(jMethod.getDescriptor());
        String[] args = new String[argumentTypes.length];
        int arg =0;
        for (Type argumentType : argumentTypes) {
            args[arg++] = argumentType.getClassName();
        }
        return args;
    }


    /**
     * Allow to get a reflect method from its metadata.
     * @param jMethod the method info
     * @param clazz the class that is containing the method
     * @return an instance of the method
     */
    @Override
    public Method getReflectMethod(final Class<?> clazz) {
        // Get method name
        String methodName = jMethod.getName();
        // Get arguments
        Type[] argumentTypes = Type.getArgumentTypes(jMethod.getDescriptor());

        // Transform string arguments into class objects
        Class<?>[] parameterTypes = null;
        if (argumentTypes != null) {
            List<Class<?>> parameters = new ArrayList<Class<?>>();
            for (Type type : argumentTypes) {
                try {
                    String parameterClassName = type.getClassName().replace("/", ".");
                    Class<?> parameterClass = null;
                    // load parameters
                    if ("int".equals(parameterClassName)) {
                        parameterClass = int.class;
                    } else if ("byte".equals(parameterClassName)) {
                        parameterClass = byte.class;
                    } else if ("char".equals(parameterClassName)) {
                        parameterClass = char.class;
                    } else if ("long".equals(parameterClassName)) {
                        parameterClass = long.class;
                    } else if ("short".equals(parameterClassName)) {
                        parameterClass = short.class;
                    } else if ("float".equals(parameterClassName)) {
                        parameterClass = float.class;
                    } else if ("double".equals(parameterClassName)) {
                        parameterClass = double.class;
                    } else if ("boolean".equals(parameterClassName)) {
                        parameterClass = boolean.class;
                    } else if (Type.ARRAY == type.getSort()) {
                        parameterClass = Class.forName(type.getDescriptor().replace('/', '.'), true, clazz.getClassLoader());
                    } else {
                        parameterClass = clazz.getClassLoader().loadClass(parameterClassName);
                    }
                    parameters.add(parameterClass);
                } catch (ClassNotFoundException e) {
                    throw new IllegalStateException("Unable to load the class '" + type.getClassName() + "'.", e);
                }
            }
            parameterTypes = parameters.toArray(new Class[parameters.size()]);
        }

        // Load the method
        try {
            return clazz.getDeclaredMethod(methodName, parameterTypes);
        } catch (SecurityException e) {
            throw new IllegalStateException("Unable to get the method '" + methodName + "'.", e);
        } catch (NoSuchMethodException e) {
            throw new IllegalStateException("Unable to get the method '" + methodName + "'.", e);
        }
    }


}
